/*
    Copyright (C) 2010 wpitchoune@gmail.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301 USA 
*/

#ifndef _PSENSOR_PSENSOR_H_
#define _PSENSOR_PSENSOR_H_

#include <sensors/sensors.h>

#include "color.h"

/* number of stored values for each sensor */
#define MAX_MEASURES 600

enum psensor_type {
	SENSOR_TYPE_TEMP = 0x01,
	SENSOR_TYPE_FAN = 0x02,
	SENSOR_TYPE_LMSENSOR_TEMP = 0x10 | SENSOR_TYPE_TEMP,
	SENSOR_TYPE_NVIDIA = 0x20 | SENSOR_TYPE_TEMP,
	SENSOR_TYPE_HDD_TEMP = 0x30 | SENSOR_TYPE_TEMP,
	SENSOR_TYPE_LMSENSOR_FAN = 0x40 | SENSOR_TYPE_FAN
};

struct psensor {
	/* Human readable name of the sensor.  It may not be uniq. */
	char *name;

	/* Uniq id of the sensor */
	char *id;

	/* lm-sensor */
	const sensors_chip_name *iname;
	const sensors_feature *feature;

	/* Last registered values of the sensor.  Index 0 for the
	   oldest value.  */
	double values[MAX_MEASURES];

	/* Color of the sensor used for the graph */
	struct color *color;

	/* Whether the sensor is displayed in the graph */
	int enabled;

	/* see psensor_type */
	int type;

	/* The maximum detected value of the sensor */
	double max;

	/* The minimum detected value of the sensor */
	double min;

	/*
	   Whether alarm alerts is enabled for this sensor
	 */
	int alarm_enabled;

	/* 
	   An alarm is raised if the current sensor value is bigger. 0
	   means no limit
	 */
	double alarm_limit;

	/* Whether the current value is bigger than 'alarm_limit'.  */
	int alarm_raised;

	void (*cb_alarm_raised) (struct psensor *, void *);
	void *cb_alarm_raised_data;

#ifdef NVIDIA_SUPPORT
	/* Nvidia id for the nvctrl */
	int nvidia_id;
#endif

};

struct psensor *psensor_create(char *id, char *name, int type);
void psensor_free(struct psensor *sensor);

void psensor_list_free(struct psensor **sensors);
int psensor_list_size(struct psensor **sensors);

/*
  Return 1 if there is at least one sensor of a given type, else
  returns 0 */
int psensor_list_contains_type(struct psensor **sensors, int type);

int is_temp_type(int type);
int is_fan_type(int type);

int get_min_temp(struct psensor **sensors);
int get_max_temp(struct psensor **sensors);

int get_min_rpm(struct psensor **sensors);
int get_max_rpm(struct psensor **sensors);

void update_psensor(struct psensor *sensor);

struct psensor *create_sensor(const sensors_chip_name * chip,
			      const sensors_feature * feature);

/*
  Converts the value of a sensor to a string.

  parameter 'type' is SENSOR_TYPE_LMSENSOR_TEMP, SENSOR_TYPE_NVIDIA,
  or SENSOR_TYPE_LMSENSOR_FAN
*/
char *psensor_value_to_string(int type, double value);

struct psensor **get_all_sensors();

struct psensor **psensor_list_add(struct psensor **sensors,
				  struct psensor *sensor);

void psensor_set_current_value(struct psensor *sensor, double value);

double psensor_get_current_value(struct psensor *sensor);

/*
  Returns a string representation of a psensor type.
*/
const char *psensor_type_to_str(int type);
#endif
